@@ -1,97 +1,180 @@
 import datetime
 import importlib
 import inspect
-from pydoc import locate
-from typing import Any, Dict, Iterable, List, Optional, Tuple
+from importlib import util as importlib_util
+from typing import Any, Dict, List, Optional, Tuple
 
 import numpy as np
 import pandas as pd
 import six
 from ciso8601 import parse_datetime
+from triad.utils.assertion import assert_or_throw
+from triad.utils.string import assert_triad_var_name
 
 EMPTY_ARGS: List[Any] = []
 EMPTY_KWARGS: Dict[str, Any] = {}
 
 
+def get_caller_global_local_vars(
+    global_vars: Optional[Dict[str, Any]] = None,
+    local_vars: Optional[Dict[str, Any]] = None,
+) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+    """Get the caller level global and local variables.
+
+    :param global_vars: overriding global variables, if not None,
+      will return this instead of the caller's globals(), defaults to None
+    :param local_vars: overriding local variables, if not None,
+      will return this instead of the caller's locals(), defaults to None
+    :return: tuple of `global_vars` and `local_vars`
+
+    :Examples:
+
+        .. code-block:: python
+            def caller():
+                x=1
+                assert 1 == get_value("x")
+
+            def get_value(var_name):
+                _, l = get_caller_global_local_vars()
+                assert var_name in l
+                assert var_name not in locals()
+                return l[var_name]
+
+    :Notice:
+
+    This is for internal use, users normally should not call this directly.
+    The concept of this function is very tricky, be careful.
+    """
+    cf = inspect.currentframe()
+    if global_vars is None:
+        global_vars = cf.f_back.f_back.f_globals  # type: ignore
+    if local_vars is None:
+        local_vars = cf.f_back.f_back.f_locals  # type: ignore
+    return global_vars, local_vars  # type: ignore
+
+
+def str_to_object(
+    expr: str,
+    global_vars: Optional[Dict[str, Any]] = None,
+    local_vars: Optional[Dict[str, Any]] = None,
+) -> Any:
+    """Convert string expression to object. The string expression must express
+    a type with relative or full path, or express a local or global instance without
+    brackets or operators.
+
+    :param expr: string expression, see examples below
+    :param global_vars: overriding global variables, if None, it will
+      use the caller's globals(), defaults to None
+    :param local_vars: overriding local variables, if None, it will
+      use the caller's locals(), defaults to None
+    :return: the
+
+    :raises ValueError: unable to find a matching object
+
+    :Examples:
+
+        .. code-block:: python
+            class _Mock(object):
+                def __init__(self, x=1):
+                    self.x = x
+
+            m = _Mock()
+            assert 1 == str_to_object("m.x")
+            assert 1 == str_to_object("m2.x", local_vars={"m2": m})
+            assert RuntimeError == str_to_object("RuntimeError")
+            assert _Mock == str_to_object("_Mock")
+
+    :Notice:
+
+    This function is to dynamically load an object from string expression.
+    If you write that string expression as python code at the same location, it
+    should generate the same result.
+    """
+    try:
+        for p in expr.split("."):
+            assert_triad_var_name(p)
+        _globals, _locals = get_caller_global_local_vars(global_vars, local_vars)
+        if "." not in expr:
+            return eval(expr, _globals, _locals)
+        root = expr.split(".")[0]
+        if root not in _globals and root not in _locals:
+            spec = importlib_util.find_spec(root)
+            assert_or_throw(spec is not None, ValueError(expr))
+            _locals = dict(_locals)
+            _locals[root] = importlib.import_module(root)
+        return eval(expr, _globals, _locals)
+    except ValueError:
+        raise
+    except Exception:
+        raise ValueError(expr)
+
+
 def str_to_type(
-    s: str, expected_base_type: Optional[type] = None, first: bool = False
+    s: str,
+    expected_base_type: Optional[type] = None,
+    global_vars: Optional[Dict[str, Any]] = None,
+    local_vars: Optional[Dict[str, Any]] = None,
 ) -> type:
     """Given a string expression, find the first/last type from all import libraries.
-    If the expression contains `.`, it's supposed to be a full path of the type
-    including modules, and this function will use `pydoc.locate` to find the type.
-    Otherwise, it will firstly check if it is a built in type, then search
-    on the call stack.
+    If the expression contains `.`, it's supposed to be a relative or full path of
+    the type including modules.
 
     :param s: type expression, for example `triad.utils.iter.Slicer` or `str`
     :param expected_base_type: base class type that must satisfy, defaults to None
-    :param first: to return the nearest (True) or farthest (False) matching one
+    :param global_vars: overriding global variables, if None, it will
+      use the caller's globals(), defaults to None
+    :param local_vars: overriding local variables, if None, it will
+      use the caller's locals(), defaults to None
 
     :raises TypeError: unable to find a matching type
 
     :return: found type
     """
-    t = list(str_to_types(s, expected_base_type))
-    if len(t) == 0:
-        if expected_base_type is not None:
-            raise TypeError(f"{s} is not type {str(expected_base_type)}")
-        else:
-            raise TypeError(f"{s} is not type")
-    if first:
-        return t[0]
-    return t[-1]
-
-
-def str_to_types(s: str, expected_base_type: Optional[type] = None) -> Iterable[type]:
-    """Given a string expression, find the type from all import libraries.
-    If the expression contains `.`, it's supposed to be a full path of the type
-    including modules, and this function will use `pydoc.locate` to find the type.
-    Otherwise, it will firstly check if it is a built in type, then search
-    on the call stack, starting from nearest level.
-
-    :param s: type expression, for example `triad.utils.iter.Slicer` or `str`
-    :param expected_base_type: base class type that must satisfy, defaults to None
-
-    :yield: found types, starting from the top on call stack
-    """
-    if "." in s:
-        t = locate(s)
-        if isinstance(t, type) and (
-            expected_base_type is None or issubclass(t, expected_base_type)
-        ):
-            yield t
-    else:
-        for tt in str_to_types("builtins." + s, expected_base_type):
-            yield tt
-        for frm in inspect.stack():
-            t = frm[0].f_globals["__name__"] + "." + s
-            for tt in str_to_types(str(t), expected_base_type):
-                yield tt
+    global_vars, local_vars = get_caller_global_local_vars(global_vars, local_vars)
+    try:
+        obj = str_to_object(s, global_vars, local_vars)
+    except ValueError:
+        raise TypeError(f"{s} is not a type")
+    assert_or_throw(isinstance(obj, type), TypeError(f"{obj} is not a type"))
+    assert_or_throw(
+        expected_base_type is None or issubclass(obj, expected_base_type),
+        TypeError(f"{obj} is not a subtype of {expected_base_type}"),
+    )
+    return obj
 
 
 def str_to_instance(
     s: str,
     expected_base_type: Optional[type] = None,
-    first: bool = False,
     args: List[Any] = EMPTY_ARGS,
     kwargs: Dict[str, Any] = EMPTY_KWARGS,
+    global_vars: Optional[Dict[str, Any]] = None,
+    local_vars: Optional[Dict[str, Any]] = None,
 ) -> Any:
     """Use :func:'~triad.utils.convert.str_to_type' to find a matching type
     and instantiate
 
     :param s: see :func:'~triad.utils.convert.str_to_type'
     :param expected_base_type: see :func:'~triad.utils.convert.str_to_type'
-    :param first: see :func:'~triad.utils.convert.str_to_type'
     :param args: args to instantiate the type
     :param kwargs: kwargs to instantiate the type
+    :param global_vars: overriding global variables, if None, it will
+      use the caller's globals(), defaults to None
+    :param local_vars: overriding local variables, if None, it will
+      use the caller's locals(), defaults to None
 
     :return: the instantiated the object
     """
-    t = str_to_type(s, expected_base_type, first)
+    global_vars, local_vars = get_caller_global_local_vars(global_vars, local_vars)
+    t = str_to_type(s, expected_base_type, global_vars, local_vars)
     return t(*args, **kwargs)
 
 
 def to_type(
-    s: Any, expected_base_type: Optional[type] = None, first: bool = False
+    s: Any,
+    expected_base_type: Optional[type] = None,
+    global_vars: Optional[Dict[str, Any]] = None,
+    local_vars: Optional[Dict[str, Any]] = None,
 ) -> type:
     """Convert an object `s` to `type`
     * if `s` is `str`: see :func:'~triad.utils.convert.str_to_type'
@@ -100,14 +183,18 @@ def to_type(
 
     :param s: see :func:'~triad.utils.convert.str_to_type'
     :param expected_base_type: see :func:'~triad.utils.convert.str_to_type'
-    :param first: see :func:'~triad.utils.convert.str_to_type'
+    :param global_vars: overriding global variables, if None, it will
+      use the caller's globals(), defaults to None
+    :param local_vars: overriding local variables, if None, it will
+      use the caller's locals(), defaults to None
 
     :raises TypeError: if no matching type found
 
     :return: the matching type
     """
+    global_vars, local_vars = get_caller_global_local_vars(global_vars, local_vars)
     if isinstance(s, str):
-        return str_to_type(s, expected_base_type, first)
+        return str_to_type(s, expected_base_type, global_vars, local_vars)
     if isinstance(s, type):
         if expected_base_type is None or issubclass(s, expected_base_type):
             return s
@@ -121,28 +208,33 @@ def to_type(
 def to_instance(
     s: Any,
     expected_base_type: Optional[type] = None,
-    first: bool = False,
     args: List[Any] = EMPTY_ARGS,
     kwargs: Dict[str, Any] = EMPTY_KWARGS,
+    global_vars: Optional[Dict[str, Any]] = None,
+    local_vars: Optional[Dict[str, Any]] = None,
 ) -> Any:
     """If s is str or type, then use :func:'~triad.utils.convert.to_type' to find
     matching type and instantiate. Otherwise return s if it matches constraints
 
     :param s: see :func:'~triad.utils.convert.to_type'
     :param expected_base_type: see :func:'~triad.utils.convert.to_type'
-    :param first: see :func:'~triad.utils.convert.to_type'
     :param args: args to instantiate the type
     :param kwargs: kwargs to instantiate the type
+    :param global_vars: overriding global variables, if None, it will
+      use the caller's globals(), defaults to None
+    :param local_vars: overriding local variables, if None, it will
+      use the caller's locals(), defaults to None
 
     :raises ValueError: if s is an instance but not a (sub)type of `expected_base_type`
     :raises TypeError: if s is an instance, args and kwargs must be empty
 
     :return: the instantiated object
     """
+    global_vars, local_vars = get_caller_global_local_vars(global_vars, local_vars)
     if s is None:
         raise ValueError("None can't be converted to instance")
     if isinstance(s, (str, type)):
-        t = to_type(s, expected_base_type, first)
+        t = to_type(s, expected_base_type, global_vars, local_vars)
         return t(*args, **kwargs)
     else:
         if expected_base_type is not None and not isinstance(s, expected_base_type):
@@ -152,59 +244,34 @@ def to_instance(
         return s
 
 
-def to_function(s: Any, first: bool = False) -> Any:  # noqa: C901
-    """For an expression, it will try to find the nearest or farthest
-    correspondent function in all imported libraries.
-    Otherwise if it is a callable and not a class, it will return the input itself.
+def to_function(
+    func: Any,
+    global_vars: Optional[Dict[str, Any]] = None,
+    local_vars: Optional[Dict[str, Any]] = None,
+) -> Any:  # noqa: C901
+    """For an expression, it tries to find the matching function.
 
     :params s: a string expression or a callable
-    :param first: to return the nearest (True) or farthest (False) matching one
+    :param global_vars: overriding global variables, if None, it will
+      use the caller's globals(), defaults to None
+    :param local_vars: overriding local variables, if None, it will
+      use the caller's locals(), defaults to None
 
     :raises AttributeError: if unable to find such a function
 
-    :return: the nearest (first=True) or farthest (False) matching function
+    :return: the matching function
     """
-    funcs = list(to_functions(s))
-    if len(funcs) == 0:
-        raise AttributeError(f"{s} is not a valid function")
-    if first:
-        return funcs[0]
-    return funcs[-1]
-
-
-def to_functions(s: Any) -> Iterable[Any]:  # noqa: C901
-    """For an expression, it will try to find all correspondent function
-    in all imported libraries, starting from the nearest on call stack.
-    Otherwise if it is a callable and not a class, it will return the input itself.
-
-    :params s: a string expression or a callable
-
-    :raises AttributeError: if unable to find such a function
-
-    :yield: matching functions
-    """
-    if s is None:
-        raise AttributeError("None can't be converted to function")
-    func: Any = None
-    if isinstance(s, str):
-        if "." in s:
-            mod_name, func_name = s.rsplit(".", 1)
-            mod = importlib.import_module(mod_name)
-            try:
-                func = getattr(mod, func_name)
-            except AttributeError:
-                func = None
-        else:
-            for f in to_functions("builtins." + s):
-                yield f
-            for frm in inspect.stack():
-                t = frm[0].f_globals["__name__"] + "." + s
-                for f in to_functions(str(t)):
-                    yield f
-    else:
-        func = s
-    if callable(func) and not isinstance(func, six.class_types):
-        yield func
+    if isinstance(func, str):
+        global_vars, local_vars = get_caller_global_local_vars(global_vars, local_vars)
+        try:
+            func = str_to_object(func, global_vars, local_vars)
+        except ValueError:
+            raise AttributeError(f"{func} is not a function")
+    assert_or_throw(
+        callable(func) and not isinstance(func, six.class_types),
+        AttributeError(f"{func} is not a function"),
+    )
+    return func
 
 
 def get_full_type_path(obj: Any) -> str:
